@@ -2,7 +2,7 @@ from __future__ import annotations
 import asyncio
 from async_timeout import timeout as atimeout
 import logging
-from typing import List, Optional
+from typing import Optional
 
 from .base_manager import BaseManager
 from .commands import BaseCommand, RC, RT
@@ -12,7 +12,6 @@ from .events import (
     build_message_map,
     ConnectionStateChangedEvent,
     EventBus,
-    MessageReceivedEvent,
     ServerReconnectedEvent,
     SessionDestroyedEvent,
     SessionInitializedEvent,
@@ -55,6 +54,7 @@ class SoulSeekClient:
             shares_cache: Optional[SharesCache] = None, transfer_cache: Optional[TransferCache] = None,
             executor_factory: Optional[ExecutorFactory] = None,
             event_bus: Optional[EventBus] = None):
+
         self.settings: Settings = settings
 
         self._stop_event: Optional[asyncio.Event] = None
@@ -81,7 +81,7 @@ class SoulSeekClient:
         self.searches: SearchManager = self.create_search_manager()
         self.server_manager: ServerManager = self.create_server_manager()
 
-        self.services: List[BaseManager] = [
+        self.services: list[BaseManager] = [
             self.users,
             self.rooms,
             self.interests,
@@ -99,8 +99,6 @@ class SoulSeekClient:
         return asyncio.get_running_loop()
 
     def register_listeners(self):
-        self.events.register(
-            MessageReceivedEvent, self._on_message_received)
         self.events.register(
             ConnectionStateChangedEvent, self._on_connection_state_changed)
         self.events.register(
@@ -227,13 +225,14 @@ class SoulSeekClient:
     async def __call__(
             self, command: BaseCommand[RC, Optional[RT]], response: bool = False,
             timeout: float = DEFAULT_COMMAND_TIMEOUT) -> Optional[RT]:
+
         return await self.execute(command, response=response, timeout=timeout)
 
     async def execute(
             self, command: BaseCommand[RC, Optional[RT]], response: bool = False,
             timeout: float = DEFAULT_COMMAND_TIMEOUT) -> Optional[RT]:
-        """Execute a `BaseCommand`, see the `commands.py` module for a list of
-        possible commands.
+        """Execute a :class:`BaseCommand`, see the :mod:`aioslsk.commands`
+        module for a list of possible commands.
 
         Waiting for a response is optional; the protocol does not always send
         error messages in case of failure. In these cases this method will
@@ -258,8 +257,8 @@ class SoulSeekClient:
 
         :param command: Command class to execute
         :param response: Whether to wait for the response or simply send
-            (default: `False`)
-        :param timeout: Timeout waiting for response (default: `10`)
+            (default: ``False``)
+        :param timeout: Timeout waiting for response (default: ``10``)
         :raise InvalidSessionError: When no logon has been performed
         :return: Optional response depending on how the command was configured
         """
@@ -366,11 +365,6 @@ class SoulSeekClient:
             self.network
         )
 
-    async def _on_message_received(self, event: MessageReceivedEvent):
-        message = event.message
-        if message.__class__ in self._MESSAGE_MAP:
-            await self._MESSAGE_MAP[message.__class__](message, event.connection)
-
     async def _on_connection_state_changed(self, event: ConnectionStateChangedEvent):
         if isinstance(event.connection, ServerConnection):
             if event.state == ConnectionState.CLOSED:
