@@ -79,10 +79,8 @@ class PeerConnectionState(Enum):
     """No init message has been received yet (PeerInit, PeerPierceFirewall)"""
     ESTABLISHED = auto()
     """Message connections: ready to receive / send data"""
-    AWAITING_TICKET = auto()
-    """Transfer connections: awaiting the transfer ticket"""
-    AWAITING_OFFSET = auto()
-    """Transfer connections: awaiting the transfer offset"""
+    NEGOTIATING_TRANSFER = auto()
+    """Transfer connections: negotiating the transfer ticket / offset"""
     TRANSFERRING = auto()
     """Transfer connections: ready to receive / send data"""
 
@@ -163,9 +161,10 @@ class ListeningConnection(Connection):
                 start_serving=True
             )
 
-        except OSError:
+        except OSError as exc:
+            adapter.exception("failed to bind listening port", extra=self.__dict__)
             await self.disconnect(CloseReason.CONNECT_FAILED)
-            raise ConnectionFailedError(f"{self.hostname}:{self.port} : failed to connect")
+            raise ConnectionFailedError(f"{self.hostname}:{self.port} : failed to connect") from exc
 
         await self.set_state(ConnectionState.CONNECTED)
 
@@ -302,8 +301,9 @@ class DataConnection(Connection, abc.ABC):
             except ConnectionReadError:
                 adapter.warning("read error", extra=self.__dict__)
 
-            except MessageDeserializationError:
-                adapter.warning("failed to deserialize message : %s", message, extra=self.__dict__)
+            except MessageDeserializationError as exc:
+                adapter.warning(
+                    "failed to deserialize message : %s", exc.proto_message, extra=self.__dict__)
 
             else:
 
@@ -532,7 +532,7 @@ class DataConnection(Connection, abc.ABC):
         try:
             message = self.deserialize_message(data)
         except Exception as exc:
-            raise MessageDeserializationError("failed to deserialize message") from exc
+            raise MessageDeserializationError(data, "failed to deserialize message") from exc
 
         return message
 
