@@ -1,7 +1,6 @@
 from collections import OrderedDict
 import logging
 import time
-from typing import Dict, List
 
 from ..base_manager import BaseManager
 from ..events import (
@@ -54,7 +53,7 @@ from ..network.connection import ConnectionState, ServerConnection
 from ..network.network import Network
 from ..settings import Settings
 from ..user.manager import UserManager
-from ..user.model import UserStatus
+from ..user.model import BlockingFlag, UserStatus
 
 
 logger = logging.getLogger(__name__)
@@ -66,38 +65,46 @@ class RoomManager(BaseManager):
     def __init__(
             self, settings: Settings, event_bus: EventBus,
             user_manager: UserManager, network: Network):
+
         self._settings: Settings = settings
         self._event_bus: EventBus = event_bus
         self._user_manager: UserManager = user_manager
         self._network: Network = network
 
-        self._rooms: Dict[str, Room] = dict()
+        self._rooms: dict[str, Room] = dict()
 
         self._MESSAGE_MAP = build_message_map(self)
 
         self.register_listeners()
 
     @property
-    def rooms(self) -> Dict[str, Room]:
+    def rooms(self) -> dict[str, Room]:
         return self._rooms
 
-    def get_joined_rooms(self) -> List[Room]:
+    def get_joined_rooms(self) -> list[Room]:
+        """Returns a list of all rooms currently joined"""
         return [room for room in self._rooms.values() if room.joined]
 
-    def get_private_rooms(self) -> List[Room]:
+    def get_private_rooms(self) -> list[Room]:
+        """Returns a list of all private rooms of which we are a member or
+        owner. This includes rooms that are not joined
+        """
         return [room for room in self._rooms.values() if room.private]
 
-    def get_public_rooms(self) -> List[Room]:
+    def get_public_rooms(self) -> list[Room]:
+        """Returns the list of public rooms"""
         return [room for room in self._rooms.values() if not room.private]
 
-    def get_owned_rooms(self) -> List[Room]:
+    def get_owned_rooms(self) -> list[Room]:
+        """Returns a list of which we are the owner"""
         me = self._user_manager.get_self()
         return [
             room for room in self._rooms.values()
             if room.owner == me.name
         ]
 
-    def get_operated_rooms(self) -> List[Room]:
+    def get_operated_rooms(self) -> list[Room]:
+        """Returns a list of rooms in which we have operator privileges"""
         me = self._user_manager.get_self()
         return [
             room for room in self._rooms.values()
@@ -127,13 +134,17 @@ class RoomManager(BaseManager):
     async def auto_join_rooms(self):
         """Automatically joins rooms stored in the settings"""
         rooms = self._settings.rooms.favorites
-        logger.info(f"automatically rejoining {len(rooms)} rooms")
+        logger.info("automatically rejoining %d rooms", len(rooms))
         await self._network.send_server_messages(
             *[JoinRoom.Request(room) for room in rooms]
         )
 
     @on_message(RoomChatMessage.Response)
     async def _on_chat_room_message(self, message: RoomChatMessage.Response, connection: ServerConnection):
+
+        if self._settings.users.is_blocked(message.username, BlockingFlag.ROOM_MESSAGES):
+            return
+
         user = self._user_manager.get_user_object(message.username)
         room = self.get_or_create_room(message.room)
         room_message = RoomMessage(
@@ -152,6 +163,10 @@ class RoomManager(BaseManager):
 
     @on_message(PublicChatMessage.Response)
     async def _on_public_chat_message(self, message: PublicChatMessage.Response, connection: ServerConnection):
+
+        if self._settings.users.is_blocked(message.username, BlockingFlag.ROOM_MESSAGES):
+            return
+
         room = self.get_or_create_room(message.room)
         user = self._user_manager.get_user_object(message.username)
 
@@ -282,8 +297,9 @@ class RoomManager(BaseManager):
             del room.tickers[user.name]
         except KeyError:
             logger.warning(
-                f"attempted to remove room ticker for user {user.name} in room {room.name} "
-                "but it wasn't present")
+                "attempted to remove room ticker for user %s in room %s but it wasn't present",
+                user.name, room.name
+            )
 
         await self._event_bus.emit(
             RoomTickerRemovedEvent(
@@ -296,11 +312,13 @@ class RoomManager(BaseManager):
     @on_message(TogglePrivateRoomInvites.Response)
     async def _on_private_room_toggle(
             self, message: TogglePrivateRoomInvites.Response, connection: ServerConnection):
-        logger.debug(f"private rooms enabled : {message.enabled}")
+
+        logger.debug("private rooms enabled : %s", message.enabled)
 
     @on_message(PrivateRoomGrantMembership.Response)
     async def _on_private_room_add_user(
             self, message: PrivateRoomGrantMembership.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         user = self._user_manager.get_user_object(message.username)
         room.members.add(message.username)
@@ -316,6 +334,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomMembershipGranted.Response)
     async def _on_private_room_added(
             self, message: PrivateRoomMembershipGranted.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         user = self._user_manager.get_self()
         room.members.add(user.name)
@@ -330,6 +349,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomRevokeMembership.Response)
     async def _on_private_room_remove_user(
             self, message: PrivateRoomRevokeMembership.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         user = self._user_manager.get_user_object(message.username)
 
@@ -347,6 +367,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomMembershipRevoked.Response)
     async def _on_private_room_removed(
             self, message: PrivateRoomMembershipRevoked.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         user = self._user_manager.get_self()
         room.members.discard(user.name)
@@ -375,6 +396,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomOperators.Response)
     async def _on_private_room_operators(
             self, message: PrivateRoomOperators.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         room.operators = set(message.usernames)
 
@@ -389,6 +411,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomOperatorGranted.Response)
     async def _on_operator_granted(
             self, message: PrivateRoomOperatorGranted.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         room.operators.discard(self._user_manager.get_self().name)
 
@@ -402,6 +425,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomOperatorRevoked.Response)
     async def _on_operator_revoked(
             self, message: PrivateRoomOperatorRevoked.Response, connection: ServerConnection):
+
         room = self.get_or_create_room(message.room, private=True)
         room.operators.discard(self._user_manager.get_self().name)
 
@@ -415,6 +439,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomGrantOperator.Response)
     async def _on_user_operator_granted(
             self, message: PrivateRoomGrantOperator.Response, connection: ServerConnection):
+
         user = self._user_manager.get_user_object(message.username)
         room = self.get_or_create_room(message.room, private=True)
         room.operators.add(message.username)
@@ -430,6 +455,7 @@ class RoomManager(BaseManager):
     @on_message(PrivateRoomRevokeOperator.Response)
     async def _on_user_operator_revoked(
             self, message: PrivateRoomRevokeOperator.Response, connection: ServerConnection):
+
         user = self._user_manager.get_user_object(message.username)
         room = self.get_or_create_room(message.room, private=True)
         room.operators.discard(message.username)
