@@ -927,8 +927,7 @@ class TransferManager(BaseManager):
         try:
             connection = await self._network.create_peer_connection(
                 transfer.username,
-                PeerConnectionType.FILE,
-                initial_state=PeerConnectionState.AWAITING_OFFSET
+                PeerConnectionType.FILE
             )
 
         except PeerConnectionError:
@@ -944,8 +943,6 @@ class TransferManager(BaseManager):
             await transfer.state.queue()
             return
 
-        connection.set_connection_state(PeerConnectionState.AWAITING_OFFSET)
-
         # Receive transfer offset
         try:
             transfer.bytes_transfered = await connection.receive_transfer_offset()
