@@ -2,7 +2,9 @@
 from aiofiles import os as asyncos
 import asyncio
 from enum import Enum
+import inspect
 import logging
+from types import MethodType
 from typing import Optional, Protocol, TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -12,17 +14,27 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
+def _with_state_lock(func):
+
+    async def wrapper(obj: 'TransferState', *args, **kwargs):
+        async with obj.transfer._state_lock:
+            result = await func(*args, **kwargs)
+        return result
+
+    return wrapper
+
+
 async def _remove_local_file(transfer: 'Transfer'):
     if not transfer.is_download():
         return
 
     if transfer.local_path:
-        logger.info(f"removing file at {transfer.local_path}")
+        logger.info("removing file from filesystem : %s", transfer.local_path)
         try:
             if await asyncos.path.exists(transfer.local_path):
                 await asyncos.remove(transfer.local_path)
         except OSError:
-            logger.warning(f"failed to remove file during abort : {transfer.local_path}")
+            logger.warning("failed to remove file during abort : %s", transfer.local_path)
 
         transfer.local_path = None
 
@@ -70,6 +82,7 @@ class TransferState:
 
     def __init__(self, transfer: 'Transfer'):
         self.transfer: 'Transfer' = transfer
+        self._wrap_lock()
 
     @classmethod
     def init_from_state(cls, state: State, transfer: 'Transfer'):
@@ -79,47 +92,64 @@ class TransferState:
 
         raise Exception(f"no state class for state : {state}")
 
-    async def fail(self, reason: Optional[str] = None) -> bool:
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        self._wrap_lock()
+
+    async def fail(self, reason: Optional[str] = None) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.FAILED.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.FAILED.name)
         return False
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.ABORTED.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.ABORTED.name)
         return False
 
-    async def queue(self, remotely: bool = False) -> bool:
+    async def queue(self, remotely: bool = False) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.QUEUED.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.QUEUED.name)
         return False
 
-    async def initialize(self) -> bool:
+    async def initialize(self) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.INITIALIZING.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.INITIALIZING.name)
         return False
 
-    async def complete(self) -> bool:
+    async def complete(self) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.COMPLETE.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.COMPLETE.name)
         return False
 
-    async def incomplete(self) -> bool:
+    async def incomplete(self) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.INCOMPLETE.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.INCOMPLETE.name)
         return False
 
-    async def start_transferring(self) -> bool:
+    async def start_transferring(self) -> bool:  # pragma: no cover
         new_state = self.UPLOADING if self.transfer.is_upload() else self.DOWNLOADING
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {new_state.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, new_state.name)
         return False
 
-    async def pause(self) -> bool:
+    async def pause(self) -> bool:  # pragma: no cover
         logger.warning(
-            f"attempted to make undefined state transition from {self.VALUE.name} to {self.PAUSED.name}")
+            "attempted to make undefined state transition from %s to %s", self.VALUE.name, self.PAUSED.name)
         return False
 
+    def _wrap_lock(self):
+        """Decorates all methods with a lock"""
+        for name, method in inspect.getmembers(self, predicate=inspect.ismethod):
+            if not name.startswith('_'):
+                setattr(self, name, MethodType(_with_state_lock(method), self))
+
+    async def _cancel_transfer_tasks(self):
+        await asyncio.gather(*self.transfer.cancel_tasks(), return_exceptions=True)
+
+    async def _stop_transfer(self):
+        await self._cancel_transfer_tasks()
+        self.transfer.set_complete_time()
+
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}({self.transfer!r})"
 
@@ -139,33 +169,26 @@ class VirginState(TransferState):
 
 
 class QueuedState(TransferState):
-    """Transfer is queued
-
-    Possible transitions:
-    - Initializing: Uploads, requesting the peer if upload is allowed
-    - Aborted: We have aborted the transfer
-    - Failed:
-        - Download: peer explicitly rejected our queue request
-        - Upload: peer explicitly rejected our transfer request
-        - Upload: peer requested a file which is not shared
-    """
     VALUE = TransferState.QUEUED
 
     async def initialize(self) -> bool:
         await self.transfer.transition(InitializingState(self.transfer))
         return True
 
-    async def fail(self, reason=None) -> bool:
+    async def fail(self, reason: Optional[str] = None) -> bool:
         self.transfer.fail_reason = reason
         await self.transfer.transition(FailedState(self.transfer))
         return True
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
+        await self._cancel_transfer_tasks()
         await _remove_local_file(self.transfer)
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
     async def pause(self) -> bool:
+        await self._cancel_transfer_tasks()
         await self.transfer.transition(PausedState(self.transfer))
         return True
 
@@ -177,39 +200,34 @@ class InitializingState(TransferState):
     peer to start uploading a file
     - Downloads: The download will quickly go into this state when the transfer
     ticket has been received over a file connection
-
-    Possible transitions:
-    - UploadingState: Upload, transfer has started
-    - QueueState:
-        - Failed to send PeerTransferRequest message
-        - Timeout receiving PeerTransferReply message
-        - Failed to make file connection
-        - Timeout waiting for transfer offset
-        - Failed to send transfer ticket
-    - AbortedState: We have aborted the transfer
-    - FailedState:
-        - PeerTransferReply was not allowed
     """
     VALUE = TransferState.INITIALIZING
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
+        await self._cancel_transfer_tasks()
         await _remove_local_file(self.transfer)
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
+    async def pause(self) -> bool:
+        await self._cancel_transfer_tasks()
+        await self.transfer.transition(PausedState(self.transfer))
+        return True
+
     async def queue(self, remotely: bool = False):
         self.transfer.remotely_queued = remotely
         await self.transfer.transition(QueuedState(self.transfer))
         return True
 
-    async def fail(self, reason=None) -> bool:
+    async def fail(self, reason: Optional[str] = None) -> bool:
         self.transfer.fail_reason = reason
         await self.transfer.transition(FailedState(self.transfer))
         return True
 
     async def start_transferring(self) -> bool:
-        self.transfer.remotely_queued = False
         self.transfer.set_start_time()
+        self.transfer.reset_queue_vars()
         if self.transfer.is_upload():
             await self.transfer.transition(UploadingState(self.transfer))
         else:
@@ -218,23 +236,8 @@ class InitializingState(TransferState):
 
 
 class DownloadingState(TransferState):
-    """
-    Possible transitions:
-    - CompleteState: Transfer has successfully completed
-    - IncompleteState:
-        - Failed to send transfer offset
-        - Connection was closed before all bytes were transfered
-    - FailedState:
-        - Could not open local file
-        - Received PeerUploadFailed message from peer
-    - Aborted: We have aborted the transfer
-    """
     VALUE = TransferState.DOWNLOADING
 
-    async def _stop_transfer(self):
-        await asyncio.gather(*self.transfer.cancel_tasks(), return_exceptions=True)
-        self.transfer.set_complete_time()
-
     async def fail(self, reason: Optional[str] = None) -> bool:
         self.transfer.fail_reason = reason
         self.transfer.set_complete_time()
@@ -246,39 +249,27 @@ class DownloadingState(TransferState):
         await self.transfer.transition(CompleteState(self.transfer))
         return True
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
         await self._stop_transfer()
         await _remove_local_file(self.transfer)
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
-    async def incomplete(self) -> bool:
-        self.transfer.set_complete_time()
-        await self.transfer.transition(IncompleteState(self.transfer))
-        return True
-
     async def pause(self) -> bool:
         await self._stop_transfer()
         await self.transfer.transition(PausedState(self.transfer))
         return True
 
+    async def incomplete(self) -> bool:
+        self.transfer.set_complete_time()
+        await self.transfer.transition(IncompleteState(self.transfer))
+        return True
+
 
 class UploadingState(TransferState):
     VALUE = TransferState.UPLOADING
 
-    """
-    Possible transitions:
-    - CompleteState: Transfer has successfully completed
-    - FailedState:
-        - Could not open local file
-        - Received PeerUploadFailed message from peer
-    - Aborted: We have aborted the transfer
-    """
-
-    async def _stop_transfer(self):
-        await asyncio.gather(*self.transfer.cancel_tasks(), return_exceptions=True)
-        self.transfer.set_complete_time()
-
     async def fail(self, reason: Optional[str] = None) -> bool:
         self.transfer.fail_reason = reason
         self.transfer.set_complete_time()
@@ -290,9 +281,10 @@ class UploadingState(TransferState):
         await self.transfer.transition(CompleteState(self.transfer))
         return True
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
         await self._stop_transfer()
         # Don't remove file
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
@@ -303,15 +295,17 @@ class UploadingState(TransferState):
 
 
 class CompleteState(TransferState):
-    """
-    Possible transitions:
-    - QueueState: Attempt transfer again
-    """
     VALUE = TransferState.COMPLETE
 
     async def queue(self, remotely: bool = False) -> bool:
+        self.transfer.reset_time_vars()
+
+        if self.transfer.is_download():
+            self.transfer.reset_progress_vars()
+            self.transfer.reset_local_vars()
+
         self.transfer.remotely_queued = remotely
-        self.transfer.reset_times()
+
         await self.transfer.transition(QueuedState(self.transfer))
         return True
 
@@ -320,10 +314,6 @@ class IncompleteState(TransferState):
     """State only used for downloads. The transfer should enter this state if an
     error occured during transferring but there was no explicit error from the
     other peer. In this state it should be possible to retry the transfer.
-
-    Possible transitions:
-    - QueueState: when attempting to retry transfers
-    - AbortedState: We have aborted the transfer
     """
     VALUE = TransferState.INCOMPLETE
 
@@ -333,59 +323,54 @@ class IncompleteState(TransferState):
         return True
 
     async def queue(self, remotely: bool = False) -> bool:
+        self.transfer.reset_time_vars()
         self.transfer.remotely_queued = remotely
-        self.transfer.reset_times()
         await self.transfer.transition(QueuedState(self.transfer))
         return True
 
     async def initialize(self) -> bool:
-        self.transfer.reset_times()
+        self.transfer.reset_time_vars()
         await self.transfer.transition(InitializingState(self.transfer))
         return True
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
+        await self._cancel_transfer_tasks()
         await _remove_local_file(self.transfer)
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
     async def pause(self) -> bool:
+        await self._cancel_transfer_tasks()
         await self.transfer.transition(PausedState(self.transfer))
         return True
 
 
 class FailedState(TransferState):
-    """
-
-    Possible transitions:
-    - CompleteState: When re-initializing the transfer
-    - QueuedState: Attempt transfer again
-    """
     VALUE = TransferState.FAILED
 
     async def queue(self, remotely: bool = False) -> bool:
+        self.transfer.reset_time_vars()
         self.transfer.remotely_queued = remotely
-        self.transfer.reset_times()
         self.transfer.fail_reason = None
+        self.transfer.abort_reason = None
         await self.transfer.transition(QueuedState(self.transfer))
         return True
 
 
 class PausedState(TransferState):
-    """
-    Possible transitions:
-    - QueuedState: Restart the transfer
-    - AbortedState: Abort the transfer
-    """
     VALUE = TransferState.PAUSED
 
     async def queue(self, remotely: bool = False) -> bool:
+        self.transfer.reset_time_vars()
         self.transfer.remotely_queued = remotely
-        self.transfer.reset_times()
         await self.transfer.transition(QueuedState(self.transfer))
         return True
 
-    async def abort(self) -> bool:
+    async def abort(self, reason: Optional[str] = None) -> bool:
+        await self._cancel_transfer_tasks()
         await _remove_local_file(self.transfer)
+        self.transfer.abort_reason = reason
         await self.transfer.transition(AbortedState(self.transfer))
         return True
 
@@ -396,16 +381,17 @@ class PausedState(TransferState):
 
 
 class AbortedState(TransferState):
-    """
-    Possible transitions:
-    - QueuedState: Attempt transfer again
-    """
     VALUE = TransferState.ABORTED
 
     async def queue(self, remotely: bool = False) -> bool:
-        # Reset all progress if the transfer is requeued after being aborted,
-        # the file should be deleted anyway
+        self.transfer.reset_time_vars()
+
+        if self.transfer.is_download():
+            self.transfer.reset_progress_vars()
+            self.transfer.reset_local_vars()
+
+        self.transfer.fail_reason = None
+        self.transfer.abort_reason = None
         self.transfer.remotely_queued = remotely
-        self.transfer.reset_progress()
         await self.transfer.transition(QueuedState(self.transfer))
         return True
